local PANEL = {}

PANEL.bg = Color( 0, 0, 0, 255 )
PANEL.fg = Color( 255, 255, 255, 255 )
PANEL.tx = Color( 0, 0, 0, 255 )
PANEL.hbg = Color( 255, 255, 255, 255 )
PANEL.hfg = Color( 0, 0, 0, 255 )
PANEL.htx = Color( 255, 255, 255, 255 )

PANEL.font = "Default"
PANEL.text = "Button"
PANEL.flagged = nil

function PANEL:Paint()
	if self.Selected and self.Armed then
		draw.RoundedBox( 1, 2, 2, self:GetWide() - 4, self:GetTall() - 4, self.hbg )
		draw.RoundedBox( 1, 3, 3, self:GetWide() - 6, self:GetTall() - 6, self.hfg )
		draw.SimpleText( self.text, self.font, self:GetWide() / 2, self:GetTall() / 2, self.htx, TEXT_ALIGN_CENTER, TEXT_ALIGN_CENTER )
	elseif self.Selected or self.Armed then
		draw.RoundedBox( 1, 0, 0, self:GetWide(), self:GetTall(), self.hbg )
		draw.RoundedBox( 1, 1, 1, self:GetWide() - 2, self:GetTall() - 2, self.hfg )
		draw.SimpleText( self.text, self.font, self:GetWide() / 2, self:GetTall() / 2, self.htx, TEXT_ALIGN_CENTER, TEXT_ALIGN_CENTER )
	else
		draw.RoundedBox( 1, 0, 0, self:GetWide(), self:GetTall(), self.bg )
		draw.RoundedBox( 1, 1, 1, self:GetWide() - 2, self:GetTall() - 2, self.fg )
		draw.SimpleText( self.text, self.font, self:GetWide() / 2, self:GetTall() / 2, self.tx, TEXT_ALIGN_CENTER, TEXT_ALIGN_CENTER )
	end

	return true
end

function PANEL:Think()
	if self.Armed and not self.flagged then
		self.flagged = true
		if self.callback then
			local b, e = pcall( self.callback.fn, unpack( self.callback.args ), "hover", self.text, true )
			if not b then
				print( "ERROR: " .. e )
			end
		end
	end

       	if not self.Armed and self.flagged then
       		self.flagged = nil
		if self.callback then
			local b, e = pcall( self.callback.fn, unpack( self.callback.args ), "hover", self.text, false )
			if not b then
				print( "ERROR: " .. e )
			end
		end
	end
end

function PANEL:SetCFont( font )
	self.font = font
end

function PANEL:SetCText( text )
	self.text = text
end

function PANEL:GetCText()
	return self.text
end

function PANEL:SetColors( bg, fg, tx, hbg, hfg, htx )
	self.bg = bg or self.bg
	self.fg = fg or self.fg
	self.tx = tx or self.tx
	self.hbg = hbg or self.hbg
	self.hfg = hfg or self.hfg
	self.htx = htx or self.htx
end

function PANEL:SetCCommand( cmd )
	self.command = cmd
end

function PANEL:SetCallback( fn, ... )
	if type( fn ) ~= "function" then
		error( "invalid arg, expected function", 2 )
		return
	end

	self.callback = { fn = fn, args = { ... } }
end

function PANEL:DoClick( x, y )
	if self.command then
		LocalPlayer():ConCommand( self.command .. "\n" )
	end
	
	if self.callback then
		local b, e = pcall( self.callback.fn, unpack( self.callback.args ), "click", self.text, true )
		if not b then
			print( "ERROR: " .. e )
		end
	end
end

vgui.Register( "URoundButton", PANEL, "Button" )